package com.fireworksdk.bridge.flutter

import androidx.annotation.NonNull
import androidx.fragment.app.FragmentActivity
import com.fireworksdk.bridge.flutter.handler.*
import com.fireworksdk.bridge.flutter.models.FWChannelName
import com.fireworksdk.bridge.flutter.models.FWFlutterFragmentDelegate
import com.fireworksdk.bridge.flutter.models.FWViewTypeId
import com.fireworksdk.bridge.flutter.platformviewfactory.FWVideoFeedFactory
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.FlutterPlugin.FlutterPluginBinding
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.platform.PlatformViewRegistry


/** FireworkFlutterSdkPlugin */
class FireworkFlutterSdkPlugin: FlutterPlugin, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var fwFlutterSdkChannel : MethodChannel
  private lateinit var fwSdkModuleChannel : MethodChannel
  private lateinit var fwShoppingModuleChannel : MethodChannel
  private lateinit var fwLiveStreamModuleChannel : MethodChannel
  private lateinit var fwNavigatorModuleChannel : MethodChannel

  private lateinit var fwFlutterSdkHandler: FWFlutterSdkHandler

  private lateinit var binaryMessenger : BinaryMessenger
  private lateinit var platformViewRegistry : PlatformViewRegistry

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPluginBinding) {
    binaryMessenger = flutterPluginBinding.binaryMessenger
    platformViewRegistry = flutterPluginBinding.platformViewRegistry

    // FireworkFlutterSdk
    fwFlutterSdkChannel = MethodChannel(binaryMessenger, FWChannelName.FireworkFlutterSdk.rawValue)
    fwFlutterSdkHandler = FWFlutterSdkHandler(flutterPluginBinding.applicationContext, fwFlutterSdkChannel)
    fwFlutterSdkChannel.setMethodCallHandler(fwFlutterSdkHandler)

    // FireworkSdkModule
    fwSdkModuleChannel = MethodChannel(binaryMessenger, FWChannelName.FireworkSdkModule.rawValue)
    fwSdkModuleChannel.setMethodCallHandler(FWSdkModuleHandler(flutterPluginBinding.applicationContext, fwSdkModuleChannel))

    // FireworkShoppingModule
    fwShoppingModuleChannel = MethodChannel(binaryMessenger, FWChannelName.FireworkShoppingModule.rawValue)
    fwNavigatorModuleChannel = MethodChannel(binaryMessenger, FWChannelName.FireworkNavigatorModule.rawValue)

    // livestream module
    fwLiveStreamModuleChannel = MethodChannel(binaryMessenger, FWChannelName.FireworkLiveStreamModule.rawValue)
    fwLiveStreamModuleChannel.setMethodCallHandler(FWLiveStreamModuleHandler(fwLiveStreamModuleChannel))
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPluginBinding) {
    fwFlutterSdkHandler.unregisterReceiver()
    fwFlutterSdkChannel.setMethodCallHandler(null)
    fwSdkModuleChannel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    val activity = binding.activity
    if (activity is FragmentActivity) {
      platformViewRegistry.registerViewFactory(
        FWViewTypeId.FireworkVideoFeed.rawValue,
        FWVideoFeedFactory(binding.activity, binaryMessenger, fwSdkModuleChannel)
      )
    } else {
      throw ClassCastException("Please include FeedView in FragmentActivity ")
    }

    fwShoppingModuleChannel.setMethodCallHandler(FWShoppingModuleHandler(activity, fwShoppingModuleChannel))
    fwNavigatorModuleChannel.setMethodCallHandler(FWNavigatorModuleHandler(activity, fwNavigatorModuleChannel))
  }

  override fun onDetachedFromActivityForConfigChanges() {
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
  }

  override fun onDetachedFromActivity() {
    fwShoppingModuleChannel.setMethodCallHandler(null)
    fwNavigatorModuleChannel.setMethodCallHandler(null)
  }

  companion object {
    var cartFragmentDelegate: FWFlutterFragmentDelegate? = null

    var containerFragmentDelegate: FWFlutterFragmentDelegate? = null
  }

}
