package com.fireworksdk.bridge.flutter.handler

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.fireworksdk.bridge.flutter.constants.FWFlutterConstant
import com.fireworksdk.bridge.flutter.utils.FWEventUtils
import com.fireworksdk.bridge.models.weakProperty
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.Result
import java.io.Serializable


class FWFlutterSdkHandler(
    private val context: Context,
    channel: MethodChannel?
): MethodChannel.MethodCallHandler {

    private var handleResult: Result? = null
    private val methodChannel by weakProperty<MethodChannel?>(channel)

    private var broadcastReceiver: BroadcastReceiver

    init {
        val filter = IntentFilter()
        filter.addAction(FWFlutterConstant.SDK_CHANNEL_ACTION)
        broadcastReceiver = MyReceiver()
        LocalBroadcastManager.getInstance(context).registerReceiver(broadcastReceiver, filter)
    }

    fun unregisterReceiver() {
        LocalBroadcastManager.getInstance(context).unregisterReceiver(broadcastReceiver)
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        handleResult = result
        if (call.method != null) {
            val intent = Intent()
            intent.action = FWFlutterConstant.SDK_CHANNEL_ACTION
            intent.putExtra(
                FWFlutterConstant.SDK_CHANNEL_INTENT,
                mapOf("method" to call.method, "arguments" to call.arguments) as Serializable
            )
            LocalBroadcastManager.getInstance(context).sendBroadcast(intent)
            result.success(null)
        } else {
            result.notImplemented()
        }
    }

    inner class MyReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            val o = intent?.getSerializableExtra(FWFlutterConstant.SDK_CHANNEL_INTENT) as Map<String, Any?>?
            if (o != null) {
                val eventName = o["method"] as String
                val arguments = o["arguments"] as Map<String, Any?>?
                FWEventUtils.sendEvent(methodChannel, eventName, arguments)
            }
        }
    }

}