package com.fireworksdk.bridge.flutter.handler

import com.fireworksdk.bridge.flutter.models.FWLiveStreamModuleCallMethod
import com.fireworksdk.bridge.flutter.utils.FWEventUtils
import com.fireworksdk.bridge.models.FWLiveStreamMessageDetailsModel
import com.fireworksdk.bridge.models.FWLiveStreamEventDetailsModel
import com.fireworksdk.bridge.models.FWLiveStreamEventName
import com.fireworksdk.bridge.models.weakProperty
import com.fireworksdk.bridge.utils.FWLogUtils
import com.loopnow.fireworklibrary.FireworkLiveStream
import com.loopnow.fireworklibrary.FireworkLiveStreamChatDetails
import com.loopnow.fireworklibrary.FireworkLiveStreamEventDetails
import com.loopnow.fireworklibrary.FireworkLiveStreamEventListener
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

class FWLiveStreamModuleHandler(
    channel: MethodChannel?
): MethodChannel.MethodCallHandler {

    private val methodChannel by weakProperty<MethodChannel?>(channel)

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            FWLiveStreamModuleCallMethod.Init.rawValue -> {
                addFireworkLiveStreamEventListener()
                result.success(null)
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    private fun addFireworkLiveStreamEventListener() {
        FireworkLiveStream.addFireworkLiveStreamEventListener(object :
            FireworkLiveStreamEventListener {
            override fun userJoined(liveStreamDetails: FireworkLiveStreamEventDetails) {
                super.userJoined(liveStreamDetails)
                FWLogUtils.d { "userJoined: $liveStreamDetails" }
                FWEventUtils.sendLiveStreamEvent(
                    methodChannel,
                    FWLiveStreamEventName.UserJoin.rawValue,
                    FWLiveStreamEventDetailsModel(liveStreamDetails.id)
                )
            }

            override fun userLeft(liveStreamDetails: FireworkLiveStreamEventDetails) {
                super.userLeft(liveStreamDetails)
                FWLogUtils.d { "userLeft: $liveStreamDetails" }
                FWEventUtils.sendLiveStreamEvent(
                    methodChannel,
                    FWLiveStreamEventName.UserLeave.rawValue,
                    FWLiveStreamEventDetailsModel(liveStreamDetails.id)
                )
            }

            override fun userSentChat(chatDetails: FireworkLiveStreamChatDetails) {
                super.userSentChat(chatDetails)
                FWLogUtils.d { "userSentChat: $chatDetails" }
                FWEventUtils.sendLiveStreamChatEvent(
                    methodChannel,
                    FWLiveStreamEventName.UserSendChat.rawValue,
                    FWLiveStreamMessageDetailsModel(chatDetails.chatId, chatDetails.username, chatDetails.text),
                    FWLiveStreamEventDetailsModel(chatDetails.liveStreamDetails.id)
                )
            }

            override fun userSentLike(liveStreamDetails: FireworkLiveStreamEventDetails) {
                super.userSentLike(liveStreamDetails)
                FWLogUtils.d { "userSentLike: $liveStreamDetails" }
                FWEventUtils.sendLiveStreamEvent(
                    methodChannel,
                    FWLiveStreamEventName.UserSendLike.rawValue,
                    FWLiveStreamEventDetailsModel(liveStreamDetails.id)
                )
            }
        })
    }

}