package com.fireworksdk.bridge.flutter.handler

import android.app.Activity
import com.fireworksdk.bridge.flutter.models.FWNavigatorModuleCallMethod
import com.fireworksdk.bridge.flutter.pages.FWContainerActivity
import com.fireworksdk.bridge.flutter.pages.FWVideoShoppingCartActivity
import com.fireworksdk.bridge.models.weakProperty
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

class FWNavigatorModuleHandler(
    currentActivity: Activity?,
    channel: MethodChannel?
): MethodChannel.MethodCallHandler {

    private val currentActivity by weakProperty<Activity?>(currentActivity)
    private val methodChannel by weakProperty<MethodChannel?>(channel)
    private var handleResult: MethodChannel.Result? = null

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        handleResult = result
        when (call.method) {
            FWNavigatorModuleCallMethod.PushNativeContainer.rawValue -> {
                val activity = currentActivity
                activity?.startActivity(FWContainerActivity.createIntent(activity, call.arguments as Map<String, Any?>?))
                result.success(true)
            }
            FWNavigatorModuleCallMethod.PopNativeContainer.rawValue -> {
                val activity = currentActivity
                if (activity is FWContainerActivity || activity is FWVideoShoppingCartActivity) {
                    activity.finish()
                    result.success(true)
                } else {
                    result.success(false)
                }
            }
            else -> {
                result.notImplemented()
            }
        }
    }

}