package com.fireworksdk.bridge.flutter.handler

import com.fireworksdk.bridge.components.videofeed.FWVideoFeed
import com.fireworksdk.bridge.flutter.models.FWVideoFeedCallMethod
import com.fireworksdk.bridge.flutter.platformview.FWVideoFeedPlatformView
import com.fireworksdk.bridge.models.FWVideoFeedConfigModel
import com.fireworksdk.bridge.models.FWVideoPlayerConfigModel
import com.fireworksdk.bridge.utils.FWJsonUtils
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.Result

class FWVideoFeedHandler(
    videoFeedPlatformView: FWVideoFeedPlatformView,
): MethodChannel.MethodCallHandler {

    private val videoFeed: FWVideoFeed = videoFeedPlatformView.videoFeed

    private var handleResult: Result? = null

    override fun onMethodCall(call: MethodCall, result: Result) {
        handleResult = result

        when (call.method) {
            FWVideoFeedCallMethod.SetVideoFeedConfiguration.rawValue -> {
                val map: HashMap<String, Any> = call.arguments as HashMap<String, Any>
                val data = FWJsonUtils.fromJson(FWJsonUtils.toJson(map), FWVideoFeedConfigModel::class.java)
                videoFeed.setVideoFeedConfigProps(data)
                result.success(null)
            }
            FWVideoFeedCallMethod.SetVideoPlayerConfiguration.rawValue -> {
                val map: HashMap<String, Any> = call.arguments as HashMap<String, Any>
                val data = FWJsonUtils.fromJson(FWJsonUtils.toJson(map), FWVideoPlayerConfigModel::class.java)
                videoFeed.setVideoPlayerConfigProps(data)
                result.success(null)
            }
            FWVideoFeedCallMethod.Refresh.rawValue -> {
                videoFeed.refresh()
                result.success(null)
            }
            else -> {
                result.notImplemented()
            }
        }
    }
}