package com.fireworksdk.bridge.flutter.utils

import com.fireworksdk.bridge.models.*
import com.fireworksdk.bridge.utils.FWLogUtils
import com.loopnow.fireworklibrary.data.Product
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.Result

object FWEventUtils {

    fun sendInitSuccessEvent(methodChannel: MethodChannel?) {
        sendEvent(methodChannel, FWEventName.SDKIni.rawValue, HashMap<String, Any>())
    }

    fun sendInitFailedEvent(methodChannel: MethodChannel?, reason: String) {
        val err = mapOf("error" to mapOf("name" to FWSDKInitSubEventName.InitFailed.rawValue, "reason" to reason))
        FWLogUtils.e { err.toString() }
        sendEvent(methodChannel, FWEventName.SDKIni.rawValue, err)
    }

    fun sendVideoFeedClickEvent(methodChannel: MethodChannel?, info: FWVideoFeedItemDetailsModel) {
        val eventMap = mapOf("info" to mapOf(
            "index" to (info.index?:0),
            "id" to info.id,
            "duration" to (info.videoDuration?:0),
            "source" to (info.source),
        ))
        sendEvent(methodChannel, FWEventName.VideoFeedClick.rawValue, eventMap)
    }

    fun sendCustomCTAClickEvent(methodChannel: MethodChannel?, url: String?) {
        sendEvent(methodChannel, FWEventName.CustomCTAClick.rawValue, mapOf("url" to url))
    }

    fun sendVideoPlaybackEvent(methodChannel: MethodChannel?, eventName: String, info: FWVideoPlaybackDetails?) {
        val eventMap = HashMap<String, Any?>()
        eventMap["eventName"] = eventName

        val contentMap = HashMap<String, Any?>()
        if (!info?.videoId.isNullOrBlank()) {
            contentMap["videoId"] = info?.videoId
        }
        if (info?.duration != null) {
            contentMap["duration"] = info.duration.toDouble()
        }
        if (info?.caption.isNullOrBlank()) {
            contentMap["caption"] = info?.caption
        }
        if (info?.badge.isNullOrBlank()) {
            contentMap["badge"] = info?.badge
        }
        if (info?.width != null || info?.height != null) {
            val playerSizeMap = HashMap<String, Any?>()
            if (info.width != null) {
                playerSizeMap["width"] = info.width
            }
            if (info.height != null) {
                playerSizeMap["height"] = info.height
            }
            contentMap["playerSize"] = playerSizeMap
        }
        if (info?.hasCta != null) {
            contentMap["hasCTA"] = info.hasCta
        }
        eventMap["info"] = contentMap

        sendEvent(methodChannel, FWEventName.VideoPlayback.rawValue, eventMap)
    }

    fun sendVideoFeedLoadSuccessEvent(methodChannel: MethodChannel?) {
        sendEvent(methodChannel, FWFeedViewEventName.VideoFeedLoadFinished.rawValue, null)
    }

    fun sendVideoFeedLoadFailedEvent(methodChannel: MethodChannel?, name: String, reason: String) {
        val err = mapOf("name" to name, "reason" to reason)
        FWLogUtils.e { err.toString() }
        sendEvent(methodChannel, FWFeedViewEventName.VideoFeedLoadFinished.rawValue, err)
    }

    fun sendAddToCartClickEvent(methodChannel: MethodChannel?, productId: String?, unitId: String?, callback: Result? = null) {
        val eventMap = HashMap<String, Any?>()
        eventMap["productId"] = productId
        eventMap["unitId"] = unitId
        sendEvent(methodChannel, FWVideoShoppingEventName.AddToCart.rawValue, eventMap, callback)
    }

    fun sendUpdateProductsDetailsEvent(methodChannel: MethodChannel?, products: List<Product>?, callback: Result? = null) {
        val eventMap = HashMap<String, Any?>()

        val productArray = ArrayList<String?>()
        if (!products.isNullOrEmpty()) {
            for (product in products) {
                productArray.add(product.extId)
            }
        }

        eventMap["productIds"] = productArray
        sendEvent(methodChannel, FWVideoShoppingEventName.UpdateProductDetails.rawValue, eventMap, callback)
    }

    fun sendEvent(methodChannel: MethodChannel?, eventName: String, eventMap: Map<String, Any?>?, callback: Result? = null) {
        methodChannel?.invokeMethod(eventName, eventMap, callback)
    }

    // live stream
    fun sendLiveStreamEvent(methodChannel: MethodChannel?, eventName: String, info: FWLiveStreamEventDetailsModel?) {
        val eventMap = HashMap<String, Any?>()
        eventMap["eventName"] = eventName

        val contentMap = HashMap<String, Any?>()
        if (!info?.id.isNullOrBlank()) {
            contentMap["id"] = info?.id
        }
        eventMap["info"] = contentMap

        sendEvent(methodChannel, FWEventName.LiveStream.rawValue, eventMap)
    }

    fun sendLiveStreamChatEvent(methodChannel: MethodChannel?, eventName: String, message: FWLiveStreamMessageDetailsModel?, liveStream: FWLiveStreamEventDetailsModel?) {
        val eventMap = HashMap<String, Any?>()
        eventMap["eventName"] = eventName

        // chat map
        val chatMap = HashMap<String, Any?>()
        if (!message?.messageId.isNullOrBlank()) {
            chatMap["messageId"] = message?.messageId
        }
        if (!message?.username.isNullOrBlank()) {
            chatMap["username"] = message?.username
        }
        if (!message?.text.isNullOrBlank()) {
            chatMap["text"] = message?.text
        }
        eventMap["chat"] = chatMap

        // live stream map
        val liveStreamMap = HashMap<String, Any?>()
        if (!liveStream?.id.isNullOrBlank()) {
            liveStreamMap["id"] = liveStream?.id
        }
        eventMap["liveStream"] = liveStreamMap

        sendEvent(methodChannel, FWEventName.LiveStreamChat.rawValue, eventMap)
    }

}