package com.fireworksdk.bridge.utils

import android.os.Looper
import com.fasterxml.jackson.annotation.JsonInclude
import com.fasterxml.jackson.core.type.TypeReference
import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.databind.SerializationFeature
import com.fasterxml.jackson.module.kotlin.jacksonObjectMapper
import java.util.*

object FWJsonUtils {
  val mapper: ObjectMapper by lazy {
    jacksonObjectMapper()
//      .setPropertyNamingStrategy(PropertyNamingStrategy.SNAKE_CASE)
      .configure(DeserializationFeature.READ_UNKNOWN_ENUM_VALUES_AS_NULL, true)
      .configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)
      .setSerializationInclusion(JsonInclude.Include.NON_NULL)
      .configure(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS, false)
      .configure(SerializationFeature.WRITE_DATES_WITH_ZONE_ID, true)
      .setTimeZone(TimeZone.getDefault())
  }

  private fun logJsonError(e: Exception) {
    val loggedError = RuntimeException("Json Read Error!", e)
    if (Looper.myLooper() != Looper.getMainLooper()) {
      throw loggedError
    }
  }

  @JvmStatic
  fun <T> fromMap(map: Map<*, *>?, cls: Class<T>): T? {
    if (map.isNullOrEmpty()) {
      return null
    }
    try {
      return mapper.convertValue(map, cls)
    } catch (e: Exception) {
      e.printStackTrace()
      logJsonError(e)
    }
    return null
  }

  @JvmStatic
  fun <T> fromJson(json: String?, cls: Class<T>): T? {
    if (json.isNullOrBlank()) {
      return null
    }
    try {
      return mapper.readValue(json, cls)
    } catch (e: Exception) {
      e.printStackTrace()
      logJsonError(e)
    }
    return null
  }

  @JvmStatic
  fun <T> fromJson(json: String?, type: TypeReference<T>): T? {
    if (json.isNullOrBlank()) {
      return null
    }
    try {
      return mapper.readValue(json, type)
    } catch (e: Exception) {
      e.printStackTrace()
      logJsonError(e)
    }
    return null
  }

  @JvmStatic
  fun toJson(obj: Any?): String? {
    if (obj == null) {
      return null
    }
    try {
      return mapper.writeValueAsString(obj)
    } catch (e: Exception) {
      e.printStackTrace()
    }
    return null
  }
}
