package com.fireworksdk.bridge.utils

import android.content.Context
import android.graphics.Color
import com.loopnow.fireworklibrary.VideoPlayerProperties
import com.fireworksdk.bridge.constants.FWVideoPlayerConstant
import com.fireworksdk.bridge.models.FWAdBadgeConfigModel
import com.fireworksdk.bridge.models.FWVideoFeedConfigModel
import com.fireworksdk.bridge.models.FWVideoFeedTitlePosition
import com.fireworksdk.bridge.models.FWVideoPlayerConfigModel
import com.loopnow.fireworklibrary.AdLabelType
import com.loopnow.fireworklibrary.VideoFeedProperties
import com.loopnow.fireworklibrary.models.FeedTitlePosition
import com.loopnow.fireworklibrary.views.PlaylistGroupFeedView
import com.loopnow.fireworklibrary.views.VideoFeedView

object FWVideoPlayerUtils {

  var customCTAClickEnabled: Boolean = false
  var videoPlaybackEventEnabled: Boolean = false

  private var hasAssignedLaunchAppWithMute = false
  /**
   * if true, mute when open player (can be set only once)
   */
  private var launchPlayerWithMute = false
    set(value) {
      if (!hasAssignedLaunchAppWithMute) {
        hasAssignedLaunchAppWithMute = true
        VideoPlayerProperties.launchPlayerWithMute = value
        field = value
      }
    }


  fun setAdBadgeConfig(config: FWAdBadgeConfigModel?) {
    val badgeTextType = config?.badgeTextType
    when {
      badgeTextType.equals(FWVideoPlayerConstant.FW_AD_BADGE_LABEL_AD) -> {
        VideoFeedProperties.setAdLabel(AdLabelType.AD)
      }
      badgeTextType.equals(FWVideoPlayerConstant.FW_AD_BADGE_LABEL_SPONSORED) -> {
        VideoFeedProperties.setAdLabel(AdLabelType.SPONSORED)
      }
    }
  }

  fun setPlaylistGroupFeedConfig(context: Context, feedView: PlaylistGroupFeedView?, config: FWVideoFeedConfigModel?) {
    config?.customLayoutName?.let {
      val layout = context.resources.getIdentifier(it, "layout", context.packageName)
      feedView?.setCustomLayout(layout)
    }
    val backgroundColor = config?.backgroundColor
    if (!backgroundColor.isNullOrBlank()) {
      feedView?.setBackgroundColor(Color.parseColor(backgroundColor))
    } else {
      feedView?.setBackgroundColor(Color.TRANSPARENT)
    }

    feedView?.setTitleVisible(config?.title?.hidden != true)

    val titlePosition = when {
      config?.titlePosition.equals(FWVideoFeedTitlePosition.Stacked.rawValue) -> FeedTitlePosition.BELOW
      else -> FeedTitlePosition.ALIGN_BOTTOM
    }
    feedView?.setTitlePosition(titlePosition)
    VideoFeedProperties.displayAdLabel = config?.showAdBadge == true
  }

  fun setVideoFeedConfig(context: Context, feedView: VideoFeedView?, config: FWVideoFeedConfigModel?) {
    config?.customLayoutName?.let {
      val layout = context.resources.getIdentifier(it, "layout", context.packageName)
      feedView?.setCustomLayout(layout)
    }

    val backgroundColor = config?.backgroundColor
    if (!backgroundColor.isNullOrBlank()) {
      feedView?.setBackgroundColor(Color.parseColor(backgroundColor))
    } else {
      feedView?.setBackgroundColor(Color.TRANSPARENT)
    }

    feedView?.setTitleVisible(config?.title?.hidden != true)

    val titlePosition = when {
      config?.titlePosition.equals(FWVideoFeedTitlePosition.Stacked.rawValue) -> FeedTitlePosition.BELOW
      else -> FeedTitlePosition.ALIGN_BOTTOM
    }
    feedView?.setTitlePosition(titlePosition)
    VideoFeedProperties.displayAdLabel = config?.showAdBadge == true
  }

  fun setVideoPlayerConfig(config: FWVideoPlayerConfigModel?) {
    VideoPlayerProperties.share = config?.showShareButton != false
    VideoPlayerProperties.loop = config?.videoCompleteAction != FWVideoPlayerConstant.FW_VIDEO_COMPLETE_ACTION_ADVANCE_TO_NEXT
//    VideoPlayerProperties.autoPlayOnComplete = config?.videoCompleteAction != FWVideoPlayerConstant.FW_VIDEO_COMPLETE_ACTION_LOOP
    VideoPlayerProperties.fullScreenPlayer = config?.playerStyle != FWVideoPlayerConstant.FW_PLAYER_STYLE_FIT
    VideoPlayerProperties.enableOneTouchMute = config?.showMuteButton == true
    VideoPlayerProperties.enablePlayPauseControl = config?.showPlaybackButton == true
    if (config?.launchBehavior != null) {
      launchPlayerWithMute = config.launchBehavior == FWVideoPlayerConstant.FW_PLAYER_LAUNCH_BEHAVIOR_MUTE_ON_FIRST_LAUNCH
    }
  }
}
