//
//  RCTConvert+FireworkSdk.swift
//  react-native-firework-sdk
//
//  Created by Jeff on 2021/12/20.
//

import Foundation
import UIKit
import FireworkVideo

extension RCTConvert {
    static func videoPlayerContentConfiguration(_ config: [String: AnyObject]?) -> VideoPlayerContentConfiguration {
        guard let config = config else {
            return VideoPlayerContentConfiguration()
        }
        
        var finalConfig = VideoPlayerContentConfiguration()
        
        if let style = videoPlayerStyle(config["playerStyle"] as? String) {
            finalConfig.playerStyle = style
        }
        if let action = videoCompleteAction(config["videoCompleteAction"] as? String) {
            finalConfig.videoCompleteAction = action
        }
        if let showShareButton = config["showShareButton"] as? Bool {
            finalConfig.shareButton.isHidden = !showShareButton
        }
        if let showMuteButton = config["showMuteButton"] as? Bool {
            finalConfig.muteButton.isHidden = !showMuteButton
        }
        if let showPlaybackButton = config["showPlaybackButton"] as? Bool {
            finalConfig.playbackButton.isHidden = !showPlaybackButton
        }
        if let ctaStyle = ctaButtonStyle(config["ctaButtonStyle"] as? Dictionary) {
            finalConfig.ctaButton = ctaStyle
        }
        
        return finalConfig
    }
    
    static func videoPlayerStyle(_ style: String?) -> VideoPlayerContentConfiguration.VideoPlayerStyle? {
        guard let rStyle = style else {
            return nil
        }
        if rStyle == "fit" {
            return .fit
        } else {
            return .fullBleed
        }
    }
    
    static func videoCompleteAction(_ action: String?) -> VideoPlayerContentConfiguration.VideoCompleteAction? {
        guard let action = action else {
            return nil
        }
        if action == "loop" {
            return .loop
        } else {
            return .advanceToNext
        }
    }
    
    static func ctaButtonStyle(_ style: [String: AnyObject]?) -> ButtonContentConfiguration? {
        guard let rStyle = style else {
            return nil
        }
        var btnContentConfig = ButtonContentConfiguration()
        if let backgroundColor = rStyle["backgroundColor"] as? String {
            btnContentConfig.backgroundColor = backgroundColor.uicolor()
        }
        if let textColor = rStyle["textColor"] as? String {
            btnContentConfig.textColor = textColor.uicolor()
        }
        if let fontSize = rStyle["fontSize"] as? Float {
            btnContentConfig.font = UIFont.systemFont(ofSize: CGFloat(fontSize))
        }
        
        return btnContentConfig
    }
}
