//
//  FireworkNavigatorModule.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/4/18.
//

import Foundation
import Flutter
import UIKit

class FireworkNavigatorModule: NSObject, FlutterPlugin {
    var channel: FlutterMethodChannel! = nil

    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: FWChannelName.FireworkNavigatorModule.rawValue, binaryMessenger: registrar.messenger())
        let instance = FireworkNavigatorModule()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case FWNavigatorModuleMethodName.pushNativeContainer.rawValue:
            pushNativeContainer(call.arguments as? [String: Any], result: result)
        case FWNavigatorModuleMethodName.popNativeContainer.rawValue:
            popNativeContainer(result: result)
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    func pushNativeContainer(_ initialize: [String: Any]?, result: @escaping FlutterResult) {
        guard let navigatorPageProvider = SwiftFireworkFlutterSdkPlugin.navigatorPageProvider else {
            print("Not implement navigatorPageProvider!")
            result(false)
            return
        }
        
        guard let presentedVC = UIViewController.presentedViewController() else {
            print("Not found top presented view controller!")
            result(false)
            return
        }
        
        guard let flutterVC = navigatorPageProvider(initialize) else {
            print("The returned flutter view controller is nil!")
            result(false)
            return
        }

        presentedVC.present(flutterVC, animated: true) {
            result(true)
        }
    }
    
    func popNativeContainer(result: @escaping FlutterResult) {
        if let presentedVC = UIViewController.presentedViewController() as? FlutterViewController {
            presentedVC.dismiss(animated: true) {
                result(true)
            }
        } else if let cartViewController = ShoppingModulePlugin.cartViewController {
            cartViewController.navigationController?.popViewController(animated: true)
            result(true)
        } else {
            result(false)
        }
    }
}
