//
//  BroadcastAdapter.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/3/9.
//

import Foundation

class BroadcastAdapter {
    static let share = BroadcastAdapter()
    private lazy var adaptees: [UUID: (String, Any?) -> Void] = [:]
    
    func addAdaptee<AdapteeType: AnyObject>(_ adaptee: AdapteeType, invokee: @escaping (_ adaptee: AdapteeType, _ methodName: String, _ arguments: Any?) -> Void) {
        let observerID = UUID()
        let adapteeClosuer = { [weak adaptee, weak self] (methodName: String, arguments: Any?) in
            guard let adaptee = adaptee else {
                self?.adaptees[observerID] = nil
                return
            }
            
            invokee(adaptee, methodName, arguments)
        }
        
        adaptees[observerID] = adapteeClosuer
    }
    
    func invokeMethod(_ methodName: String, arguments: Any?) {
        for adaptee in adaptees.values {
            adaptee(methodName, arguments)
        }
    }
}
