//
//  DispatchQueue+Once.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/3/1.
//

import Foundation

public extension DispatchQueue {
    private static var _onceContainer = [String]()
    
    /// Executes a block of code, associated with a unique token, only once.  The code is thread safe and will only execute the code once even in the presence of multithreaded calls.
    /// - Parameters:
    ///   - file: <#file description#>
    ///   - function: <#function description#>
    ///   - line: <#line description#>
    ///   - block: <#block description#>
    class func once(file: String = #file, function: String = #function, line: Int = #line, block: ()->Void) {
        let token = file + ":" + function + ":" + String(line)
        once(token: token, block: block)
    }

    /// Executes a block of code, associated with a unique token, only once.  The code is thread safe and will only execute the code once even in the presence of multithreaded calls.
    /// - Parameters:
    ///   - token: A unique  name
    ///   - block: Block to execute once
    class func once(token: String, block: ()->Void) {
        objc_sync_enter(self)
        defer { objc_sync_exit(self) }

        guard !_onceContainer.contains(token) else {
            return
        }
        
        _onceContainer.append(token)
        block()
    }
}
