import 'dart:async';

import 'models/fw_events.dart';
import 'utils/fw_method_channel_util.dart';
import 'utils/model_utils/fw_events_util.dart';

const _eventBusMethodName = "broadcastEvent";

/// Dispatches events to all listeners.
/// You can receive the events dispatched in another flutter engine.
class FWEventBus {
  static FWEventBus? _instance;
  final StreamController _streamController = StreamController.broadcast();

  static FWEventBus _getInstance() {
    _instance ??= FWEventBus._();
    return _instance!;
  }

  FWEventBus._() {
    FWMethodChannelUtil.getEventBusMethodChannel()
        .setMethodCallHandler((call) async {
      if (call.method == _eventBusMethodName) {
        final event = FWEventUtil.fromJson(call.arguments);
        if (event != null) {
          _streamController.add(event);
        }
      }
      return null;
    });
  }

  /// Get global single instance of FWEventBus class.
  factory FWEventBus.getInstance() => _getInstance();

  /// Fires an event.
  void fire(FWEvent event) {
    FWMethodChannelUtil.getEventBusMethodChannel().invokeMethod(
      _eventBusMethodName,
      FWEventUtil.toJson(event),
    );
  }

  /// Listens for events.
  Stream<FWEvent> on() {
    return _streamController.stream
        .where((event) => event is FWEvent)
        .cast<FWEvent>();
  }
}
