import '../../models/video_playback_details.dart';

const videoPlaybackEventNameMap = <String, VideoPlaybackEventName>{
  "fw:video:impression": VideoPlaybackEventName.impression,
  "fw:video:start": VideoPlaybackEventName.start,
  "fw:video:midpoint": VideoPlaybackEventName.midpoint,
  "fw:video:third-quartile": VideoPlaybackEventName.thirdQuartile,
  "fw:video:complete": VideoPlaybackEventName.complete,
  "fw:video:ad-end": VideoPlaybackEventName.adEnd,
  "fw:video:click-cta": VideoPlaybackEventName.clickCTA,
  "fw:video:click-share": VideoPlaybackEventName.clickShare,
};

class VideoPlaybackDetailsUtil {
  static VideoPlayerSize? _createPlayerSize(dynamic json) {
    if (json is Map<dynamic, dynamic>) {
      if (json["width"] is num && json["height"] is num) {
        return VideoPlayerSize(
          width: (json["width"] as num).toDouble(),
          height: (json["height"] as num).toDouble(),
        );
      }
    }

    return null;
  }

  static VideoPlaybackDetails? fromJson(dynamic json) {
    if (json is Map<dynamic, dynamic>) {
      if (json["videoId"] is String) {
        final videoId = json["videoId"] as String;
        VideoPlaybackDetails details = VideoPlaybackDetails(videoId: videoId);
        if (json["duration"] is num) {
          details.duration = (json["duration"] as num).toDouble();
        }

        if (json["caption"] is String) {
          details.caption = json["caption"] as String;
        }

        if (json["badge"] is String) {
          details.badge = json["badge"] as String;
        }

        details.playerSize = _createPlayerSize(json["playerSize"]);

        if (json["hasCTA"] is bool) {
          details.hasCTA = json["hasCTA"] as bool;
        }

        return details;
      }
    }
    return null;
  }

  static VideoPlaybackEventName? createVideoPlaybackEventName(String rawName) {
    final eventName = videoPlaybackEventNameMap[rawName];
    return eventName;
  }
}
